import { CancellationToken } from './cancellation';
import { RemoteLanguageClient } from './lang-handler';
import { Span } from 'opentracing';
import { InMemoryFileSystem } from './memfs';
export interface FileSystem {
    /**
     * Returns all files in the workspace under base
     *
     * @param base A URI under which to search, resolved relative to the rootUri
     * @return A promise that is fulfilled with an array of URIs
     */
    getWorkspaceFiles(base?: string, token?: CancellationToken): Promise<Iterable<string>>;
    /**
     * Returns the content of a text document
     *
     * @param uri The URI of the text document, resolved relative to the rootUri
     * @return A promise that is fulfilled with the text document content
     */
    getTextDocumentContent(uri: string, token?: CancellationToken): Promise<string>;
}
export declare class RemoteFileSystem implements FileSystem {
    private client;
    constructor(client: RemoteLanguageClient);
    /**
     * The files request is sent from the server to the client to request a list of all files in the workspace or inside the directory of the base parameter, if given.
     * A language server can use the result to index files by filtering and doing a content request for each text document of interest.
     */
    getWorkspaceFiles(base?: string): Promise<Iterable<string>>;
    /**
     * The content request is sent from the server to the client to request the current content of any text document. This allows language servers to operate without accessing the file system directly.
     */
    getTextDocumentContent(uri: string): Promise<string>;
}
export declare class LocalFileSystem implements FileSystem {
    private rootPath;
    /**
     * @param rootPath The root directory path that relative URIs should be resolved to
     */
    constructor(rootPath: string);
    /**
     * Converts the URI to an absolute path
     */
    protected resolveUriToPath(uri: string): string;
    getWorkspaceFiles(base?: string): Promise<Iterable<string>>;
    getTextDocumentContent(uri: string): Promise<string>;
}
/**
 * Synchronizes a remote file system to an in-memory file system
 *
 * TODO: Implement Disposable with Disposer
 */
export declare class FileSystemUpdater {
    private remoteFs;
    private inMemoryFs;
    /**
     * Promise for a pending or fulfilled structure fetch
     */
    private structureFetch?;
    /**
     * Map from URI to Promise of pending or fulfilled content fetch
     */
    private fetches;
    /**
     * Limits concurrent fetches to not fetch thousands of files in parallel
     */
    private concurrencyLimit;
    constructor(remoteFs: FileSystem, inMemoryFs: InMemoryFileSystem);
    /**
     * Fetches the file content for the given URI and adds the content to the in-memory file system
     *
     * @param uri URI of the file to fetch
     * @param childOf A parent span for tracing
     */
    fetch(uri: string, childOf?: Span): Promise<void>;
    /**
     * Returns a promise that is resolved when the given URI has been fetched (at least once) to the in-memory file system.
     * This function cannot be cancelled because multiple callers get the result of the same operation.
     *
     * @param uri URI of the file to ensure
     * @param childOf A parent span for tracing
     */
    ensure(uri: string, childOf?: Span): Promise<void>;
    /**
     * Fetches the file/directory structure for the given directory from the remote file system and saves it in the in-memory file system
     *
     * @param childOf A parent span for tracing
     */
    fetchStructure(childOf?: Span): Promise<void>;
    /**
     * Returns a promise that is resolved as soon as the file/directory structure for the given directory has been synced
     * from the remote file system to the in-memory file system (at least once)
     *
     * @param childOf A parent span for tracing
     */
    ensureStructure(childOf?: Span): Promise<void>;
    /**
     * Invalidates the structure fetch cache.
     * The next call to `ensureStructure` will do a refetch.
     */
    invalidateStructure(): void;
}
